@@ -1,7 +1,9 @@
+var editor
+
 function init(data) {
     const container = document.getElementById("jsoneditor");
     const options = data.options;
-    const editor = new JSONEditor(container, options ? options : { colorPicker: false });
+    editor = new JSONEditor(container, options);
     const initialJson = data.data;
     const jsoneditorMenu = document.getElementsByClassName('jsoneditor-menu')[0]
     editor.set(initialJson);
@@ -12,15 +14,21 @@ function init(data) {
     if (data.keep_running) {
         addCloseButton(jsoneditorMenu);
     }
+    if (data.additional_js) {
+        eval(data.additional_js)
+    }
+    if (!data.keep_running) {
+        setTimeout(() => makeSureServerIsClosed(), 1000)
+    }
 }
 
 
-function setFavicon() {
-    const favicon = document.createElement('link');
-    favicon.rel = 'icon';
-    favicon.href = 'files/img/favicon.ico?random=' + (Math.random() + 1).toString(36).substring(7);
-    document.getElementsByTagName('head')[0].appendChild(favicon);
-};
+function makeSureServerIsClosed() {
+    let xhr = new XMLHttpRequest();
+    xhr.open("GET", window.location.origin + '/close');
+    xhr.onerror = () => {}
+    xhr.send(null);
+}
 
 
 function addCallbackButton(jsoneditorMenu, editor) {
@@ -41,8 +49,8 @@ function addCallbackButton(jsoneditorMenu, editor) {
 
 function addCloseButton(jsoneditorMenu) {
     const closeButton = document.createElement('button')
-    closeButton.className = 'finnish-and-shuttdown'
-    closeButton.title = 'Finnish and shuttdown'
+    closeButton.className = 'finish-and-shutdown'
+    closeButton.title = 'Finish and Shutdown'
     jsoneditorMenu.append(closeButton)
     closeButton.onclick = () => {
         let xhr = new XMLHttpRequest();
@@ -72,6 +80,4 @@ function getData() {
     xhr.send();
 }
 
-
-setFavicon();
 getData();
