@@ -5,12 +5,13 @@ import json
 import mimetypes
 import os
 import random
+import subprocess
 import sys
 import threading
 import webbrowser
 from collections.abc import Mapping
 from io import TextIOWrapper
-from typing import Union, Callable, Any
+from typing import Any, Callable, Union
 from urllib.parse import urlparse
 from wsgiref.simple_server import WSGIRequestHandler, make_server
 
@@ -18,17 +19,27 @@ import pyperclip
 import requests
 
 # Get installation dir
-install_dir = os.path.dirname(os.path.realpath(__file__))
+INSTALL_DIR = os.path.dirname(os.path.realpath(__file__))
+RESOURCES = [
+    "/",
+    "/files/jsoneditor.min.css",
+    "/files/index.css",
+    "/files/jsoneditor.min.js",
+    "/files/index.js",
+    "/get_data",
+    "/files/img/jsoneditor-icons.svg",
+    "/files/img/favicon.ico",
+]
 
 
 class AltWsgiHandler(WSGIRequestHandler):
-    def log_message(self, format, *args) -> None:
-        self.server.number_of_requests += 1
+    def log_message(self, _format, *args) -> None:
+        if self.path in self.server.resources:
+            self.server.resources.remove(self.path)
         if self.path == "/close" or (
-            not self.server.keep_running and self.server.number_of_requests == 8
+            not self.server.keep_running and not self.server.resources
         ):
-            self.server._BaseServer__shutdown_request = True
-            self.server.server_close()
+            self.server.stop()
 
 
 class Server:
@@ -37,26 +48,39 @@ class Server:
         data: Union[dict, str, list],
         callback: Callable[[dict], Any] = None,
         options: dict = None,
+        additional_js: str = None,
         keep_running: bool = False,
         run_in_thread: bool = False,
         is_csv: bool = False,
+        is_ndjson: bool = False,
+        is_js_object: bool = False,
         title: str = None,
         port: int = None,
+        no_browser: bool = False,
     ) -> None:
         self.callback = callback
         self.options = options
+        self.additional_js = additional_js
         self.keep_running = keep_running
         self.run_in_thread = run_in_thread
         self.is_csv = is_csv
+        self.is_ndjson = is_ndjson
+        self.is_js_object = is_js_object
         self.title = title
+        self.no_browser = no_browser
         self.get_random_port(port)
         self.data = self.get_json(data)
+        self.server = None
 
     def get_random_port(self, port: int = None):
         self.port = port or random.randint(1023, 65353)
 
-    def send_response(self, status, content_type, respond):
+    def send_response(
+        self, status: str, content_type: str, respond: Callable, cache: bool = False
+    ):
         headers = [("Content-type", content_type)]
+        if cache:
+            headers.append(("Cache-Control", "max-age=259200"))
         respond(status, headers)
 
     def get_json(self, source: Union[dict, str, list]) -> dict:
@@ -64,15 +88,13 @@ class Server:
         retrieved_data = None
         if type(source) is str:
             if self.is_url(source):
-                if source.endswith(".csv"):
-                    self.is_csv = True
+                self.detect_source_by_filename(source)
                 self.title = self.title or source.split("/")[-1].split("?")[0]
                 retrieved_data = requests.get(source).text
             elif self.is_file(source):
-                if source.endswith(".csv"):
-                    self.is_csv = True
+                self.detect_source_by_filename(source)
                 self.title = self.title or os.path.basename(source)
-                retrieved_data = open(source, "r")
+                retrieved_data = open(source, "r", encoding="utf-8")
             else:
                 retrieved_data = source
             try:
@@ -104,6 +126,12 @@ class Server:
     def is_file(source: str) -> bool:
         return os.path.exists(source)
 
+    def detect_source_by_filename(self, source: str):
+        if source.endswith(".csv"):
+            self.is_csv = True
+        elif any(source.endswith(ext) for ext in [".ndjson", ".jsonl"]):
+            self.is_ndjson = True
+
     def load_json(self, source):
         if self.is_csv:
             if isinstance(source, str):
@@ -112,6 +140,22 @@ class Server:
                 result = list(csv.DictReader(source.split("\n")))
             elif isinstance(source, TextIOWrapper):
                 result = list(csv.DictReader(source))
+        elif self.is_ndjson:
+            if isinstance(source, str):
+                lines = source.splitlines()
+            elif isinstance(source, TextIOWrapper):
+                lines = source.readlines()
+            result = list(json.loads(line) for line in lines)
+        elif self.is_js_object:
+            try:
+                node = subprocess.Popen(['node'], stdout=subprocess.PIPE, stdin=subprocess.PIPE)
+                stdout, _ = node.communicate(f"console.log(JSON.stringify({source}))".encode())
+                result = json.loads(stdout.decode())
+            except FileNotFoundError:
+                print(
+                    "You need to have Nodejs installed to be able to use JavaScript Objects."
+                )
+                sys.exit(1)
         else:
             if isinstance(source, str):
                 try:
@@ -127,12 +171,12 @@ class Server:
     def wsgi_app(self, environ, respond):
         path = environ["PATH_INFO"]
         method = environ["REQUEST_METHOD"]
-        file_path = install_dir + path
+        file_path = INSTALL_DIR + path
         # index.html
         if method == "GET":
             if path == "/":
                 self.send_response("200 OK", "text/html", respond)
-                with open(install_dir + "/files/index.html", "rb") as f:
+                with open(INSTALL_DIR + "/files/index.html", "rb") as f:
                     content = f.read()
                 yield content
             # Data endpiont
@@ -142,18 +186,19 @@ class Server:
                     "data": self.data,
                     "callback": bool(self.callback),
                     "options": self.options,
+                    "additional_js": self.additional_js,
                     "title": self.title or "jsoneditor",
                     "keep_running": self.keep_running,
                 }
-                yield json.dumps(data).encode("utf-8")
+                yield json.dumps(data, default=repr).encode("utf-8")
             # Close endpoint
             elif path == "/close":
                 self.send_response("200 OK", "text/plain", respond)
                 yield b""
             # Serve static files
             elif path.startswith("/files") and os.path.exists(file_path):
-                type = mimetypes.guess_type(file_path)[0]
-                self.send_response("200 OK", type, respond)
+                mimetype = mimetypes.guess_type(file_path)[0]
+                self.send_response("200 OK", mimetype, respond, True)
                 with open(file_path, "rb") as f:
                     content = f.read()
                 yield content
@@ -176,20 +221,27 @@ class Server:
         # We might get an error if the port is in use.
         while True:
             try:
-                server = make_server(
+                self.server = make_server(
                     "", self.port, self.wsgi_app, handler_class=AltWsgiHandler
                 )
                 break
             except OSError:
                 self.get_random_port()
 
-        server.number_of_requests = 0
-        server.keep_running = self.keep_running
+        self.server.resources = RESOURCES.copy()
+
+        self.server.keep_running = self.keep_running
+        self.server.stop = self.stop
 
         if not self.run_in_thread:
-            open_browser(self.port)
+            open_browser(self.port, self.no_browser)
 
-        server.serve_forever()
+        self.server.serve_forever()
+
+    def stop(self):
+        if self.server:
+            self.server._BaseServer__shutdown_request = True
+            self.server.server_close()
 
 
 # Entry point
@@ -197,31 +249,52 @@ def editjson(
     data: Union[dict, str, list],
     callback: Callable[[dict], Any] = None,
     options: dict = None,
+    additional_js: str = None,
     keep_running: bool = False,
     run_in_thread: bool = False,
     is_csv: bool = False,
+    is_ndjson: bool = False,
+    is_js_object: bool = False,
     title: str = None,
     port: int = None,
-) -> None:
+    no_browser: bool = False,
+) -> Server:
     keep_running = keep_running or bool(callback)
+
     server = Server(
-        data, callback, options, keep_running, run_in_thread, is_csv, title, port
+        data,
+        callback,
+        options,
+        additional_js,
+        keep_running,
+        run_in_thread,
+        is_csv,
+        is_ndjson,
+        is_js_object,
+        title,
+        port,
+        no_browser,
     )
 
     if server.run_in_thread:
         thread = threading.Thread(target=server.start)
         thread.start()
-        open_browser(server.port)
+        open_browser(server.port, no_browser)
     else:
         server.start()
 
+    return server
 
-def open_browser(port: int) -> None:
-    browser_opened = webbrowser.open(f"http://localhost:{port}/")
-    if not browser_opened:
-        print(
-            f"Couldn't launch browser, Please open this link to see the JSON: http://localhost:{port}/"
-        )
+
+def open_browser(port: int, no_browser: bool) -> None:
+    if no_browser:
+        print(f"Please open this link to see the JSON: http://localhost:{port}/")
+    else:
+        browser_opened = webbrowser.open(f"http://localhost:{port}/")
+        if not browser_opened:
+            print(
+                f"Couldn't launch browser, Please open this link to see the JSON: http://localhost:{port}/"
+            )
 
 
 # cli function
@@ -243,17 +316,24 @@ def main() -> None:
     )
     parser.add_argument(
         "-o",
-        help="Add a button that will output the json back to the console",
+        help="Add a button that will output the JSON back to the console",
         action="store_true",
     )
-    parser.add_argument("-b", help="Keep running in backround", action="store_true")
+    parser.add_argument("-b", help="Keep running in background", action="store_true")
     parser.add_argument("-c", help="Get data input from clipboard", action="store_true")
     parser.add_argument("-k", help="Keep alive", action="store_true")
     parser.add_argument("-e", help="Edit mode", action="store_true")
+    parser.add_argument("-n", help="Don't launch browser", action="store_true")
     parser.add_argument("-p", help="Server port")
     parser.add_argument("--out", help="File to output when in edit mode")
     parser.add_argument("-t", help="Title to display in browser window")
     parser.add_argument("--csv", help="Input is CSV", action="store_true")
+    parser.add_argument(
+        "--js", help="Input is a JavaScript Object", action="store_true"
+    )
+    parser.add_argument(
+        "--ndjson", help="Input is Newline Delimited JSON", action="store_true"
+    )
     args = parser.parse_args()
 
     options = {}
@@ -272,9 +352,18 @@ def main() -> None:
     if args.t:
         options["title"] = args.t
 
+    if args.n:
+        options["no_browser"] = True
+
     if args.csv:
         options["is_csv"] = True
 
+    if args.js:
+        options["is_js_object"] = True
+
+    if args.ndjson:
+        options["is_ndjson"] = True
+
     if not os.isatty(0):
         options["data"] = "".join(x for x in sys.stdin)
     else:
@@ -294,7 +383,7 @@ def main() -> None:
             elif os.path.exists(options["data"]):
                 file_path = options["data"]
             if file_path:
-                with open(file_path, "w") as f:
+                with open(file_path, "w", encoding="utf-8") as f:
                     if args.csv or options["data"].endswith(".csv"):
                         csv_fields = json_data[0].keys() if json_data else []
                         writer = csv.DictWriter(f, csv_fields)
